/* ========================================================================
A Server Connection Data Example 

The first of our ADO.NET review examples will demonstrate using a DataSet
and a DataAdapter to retrieve data from SQL Server, and post changes such
as inserts, updates, and deletes through the adapter and their appropriate
IDbCommand objects. 
======================================================================== */
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Data.SqlClient;

namespace AdoNetSample
{
	public class AdoNetSample : System.Windows.Forms.Form
	{
		public System.Windows.Forms.RichTextBox richTextBox;
		private System.ComponentModel.Container components = null;
		public  string msg;

		public AdoNetSample()
		{
			InitializeComponent();

			DataSet SampleDS = new DataSet("SampleDS");
			string ConnectionString = "Data Source=XEON; "+
				"Initial Catalog=Northwind; User id=sa; Password=manager;";
			
			SqlConnection Connection = new SqlConnection(ConnectionString);
			
			SqlDataAdapter SampleDA = new SqlDataAdapter(
				"SELECT RegionID,RegionDescription FROM Region", Connection);
			
			// Now lets provide SQL statements that support the rest of the
			// UPDATE, DELETE, and INSERT functionality.

			// It's worth noting here than when creating commands to commit
			// changes for the DataAdapter, it is important to indicate the source column.
			// You can do that by setting the SourceColumn property on the Parameter object,
			// or by passing it as the argument immediately following the parameter's data type size.

			// For example, the parameter, @RegionDescription, when the DataAdapter
			// attempted to invoke the Update command, couldn't decipher on its own that the value
			// to be supplied for @RegionDescription was the RegionDescription column.
			// By supplying the SourceColumn to the Parameters.Add() method on our own,
			// we guarantee that the adapter will supply the right column values for the right arguments.

			// -- UPDATE --
			SqlCommand UpdateCommand = new SqlCommand(
				"UPDATE Region SET RegionDescription = @RegionDescription " +
				"WHERE RegionID = @RegionID", Connection);

			SqlParameter UpdateKeyParam = new SqlParameter("@RegionID",SqlDbType.Int);
			UpdateKeyParam.SourceColumn = "RegionID";
			UpdateKeyParam.SourceVersion = DataRowVersion.Original;

			UpdateCommand.Parameters.Add("@RegionDescription", SqlDbType.NChar, 50, "RegionDescription");
			UpdateCommand.Parameters.Add(UpdateKeyParam);
			SampleDA.UpdateCommand = UpdateCommand;

			// -- DELETE --
			SqlCommand DeleteCommand = new SqlCommand(
				"DELETE Region WHERE RegionID = @RegionID", Connection);

			SqlParameter DeleteKeyParam = new SqlParameter("@RegionID",SqlDbType.Int);
			DeleteKeyParam.SourceColumn = "RegionID";
			DeleteKeyParam.SourceVersion = DataRowVersion.Original;
			
			DeleteCommand.Parameters.Add(DeleteKeyParam);
			SampleDA.DeleteCommand = DeleteCommand;

			// -- INSERT --
			SqlCommand InsertCommand = new SqlCommand(
				"INSERT INTO Region(RegionDescription, RegionID) "+
				"VALUES(@RegionDescription, @RegionID)", Connection);

			InsertCommand.Parameters.Add("@RegionDescription", SqlDbType.NChar, 50,"RegionDescription");
			InsertCommand.Parameters.Add("@RegionID", SqlDbType.Int, 4, "RegionID");
			SampleDA.InsertCommand = InsertCommand;

			// Now load the dataset with the results of our SQL Query.
			// notice that we're not explicitly opening our conection. 
			// Our DataAdapter is doing all that work for us, closing
			// it as soon as it completes its task.
			SampleDA.Fill(SampleDS, "Region");

			// Create a new row the Region table doesn't have an
			// autonumbering identity, so we have to supply our own region ID.
			DataRow NewRow;
			NewRow = SampleDS.Tables["Region"].NewRow();
			NewRow["RegionDescription"] = "Central";
			NewRow["RegionID"] = 5;

			msg = "New Row Created using NewRow(), RowState is: " + NewRow.RowState.ToString() + "\n";
			SampleDS.Tables["Region"].Rows.Add(NewRow);
			msg += "New Row Added to Table RowState is: " + NewRow.RowState.ToString() + "\n";

			// Modify the first row			
			DataRow TempRow;
			TempRow = SampleDS.Tables["Region"].Rows[0];

			msg += "Modifying First Row, Pre-Change State is: " + TempRow.RowState.ToString() + "\n";
			TempRow["RegionDescription"] = "The wonderful Region of Berne";
			msg += "Modifying First Row, Post-Change State is: " + TempRow.RowState.ToString() + "\n";

			// Call the update method to save the new row and update the first
            msg += "Calling Update() to Commit New Row and First Row Change.\n";
			SampleDA.Update(SampleDS, "Region");

			// Delete the second row
            msg += "Deleting Our New Row, Post-Delete State is: " + NewRow.RowState.ToString() + "\n";
			NewRow.Delete();
			msg += "Deleting Our New Row, Post-Delete State is: " + NewRow.RowState.ToString() + "\n";

			// Now call the update method.
            msg += "Calling Update() - this will post ALL of our changes, not just the update\n";

			SampleDA.Update(SampleDS, "Region");
            msg += "Region Table after Update()\n------------------------------\n";

			foreach (DataRow tRow in SampleDS.Tables["Region"].Rows)
			{
				msg += tRow["RegionDescription"].ToString() + "\n";
			}
			richTextBox.Text = msg;

			SampleDS.Dispose();
			SampleDA.Dispose ();
		}

		[STAThread]
		static void Main() 
		{
			Application.Run(new AdoNetSample());
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.richTextBox = new System.Windows.Forms.RichTextBox();
			this.SuspendLayout();
			// 
			// richTextBox
			// 
			this.richTextBox.Font = new System.Drawing.Font("Courier New", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.richTextBox.Location = new System.Drawing.Point(10, 8);
			this.richTextBox.Name = "richTextBox";
			this.richTextBox.Size = new System.Drawing.Size(672, 312);
			this.richTextBox.TabIndex = 0;
			this.richTextBox.Text = "";
			// 
			// AdoNetSample
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(691, 329);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.richTextBox});
			this.Name = "AdoNetSample";
			this.Text = "DataRowState";
			this.ResumeLayout(false);

		}
		#endregion
	}
}
